-- ClassicBestiary by Urbit @ Benediction
-- Inspired by https://wago.io/q1YbxB5Pz - tooltip effect code has been adapted from there.
--
-- Uses an ability database generated by scraping Wowhead for creatures identified in Questie.
-- Some abilities (namely Dazed) have been removed, and repeated abilities have been collapsed.
-- Spell tooltips may have inaccurate values, but should reflect what the spell does.

local DB = _G["ClassicBestiary"]

local hook_installed = false

local function on_tooltip_set_unit()
  local name, id = GameTooltip:GetUnit()
  local guid_frag = (UnitGUID(id)):reverse():sub(12)
  local hep_pos = guid_frag:find("-")
  local npc_id = tonumber(guid_frag:sub(1, hep_pos - 1):reverse())
  local abilities = DB.map[npc_id]

  if abilities ~= nil then
    for _, spell_id in ipairs(abilities) do
      local name, _rank, _icon, _castTime, _minRange, _maxRange = GetSpellInfo(spell_id);
      if name ~= nil then
        local icon = "";
        local texture = GetSpellTexture(spell_id);
        if (texture ~= nil) then
          icon = "|T" .. texture .. ":0|t";
        end

        local tip = DB.tip[spell_id]
        local range = ""

        if tip ~= nil then
          local text_id, range = tip[1], tip[2]

          -- text 0 is always ""
          if text_id == 0 then
            tip = GetSpellDescription(spell_id)
          else
            tip = DB.st[text_id]
          end

          if (range == 0) then
            range = "Melee range"
          elseif (range == -1) then
            range = ""
          elseif (range == -2) then
            range = "Unlimited range"
          else
            range = tostring(range) .. " yd range"
          end
        else
          tip = GetSpellDescription(spell_id)
        end

        GameTooltip:AddLine(icon .. " " .. name .. " " .. range)
        if tip ~= nil and IsControlKeyDown() then
          GameTooltip:AddLine(tip, 0.8, 0.8, 0.8, true)
        end
      end
    end

    if not IsControlKeyDown() then
      GameTooltip:AddLine("(Ctrl for details)", 0.8, 0.8, 0.8)
    end

    if GameTooltip:GetWidth() > 700 then
      GameTooltip:SetWidth(700)
    end
  end
end

local function on_event(_frame, e, ...)
  if e == "PLAYER_ENTERING_WORLD" then
    if not hook_installed then
        GameTooltip:HookScript("OnTooltipSetUnit", on_tooltip_set_unit)
        hook_installed = true
    end
  elseif e == "MODIFIER_STATE_CHANGED" then
    if UnitExists("mouseover") then
      GameTooltip:SetUnit("mouseover");
    end
  end
end

local frame = CreateFrame("Frame", "ClassicBestiaryEvents")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("MODIFIER_STATE_CHANGED")
frame:SetScript("OnEvent", on_event)
